//
// Header for graphics surfaces.
//
//! \file tonc_surface.h
//! \author J Vijn
//! \date 20080119 - 20080119
//
/* === NOTES ===
*/


#ifndef TONC_SURFACE
#define TONC_SURFACE

#include "tonc_memmap.h"
#include "tonc_core.h"

/*! \defgroup	grpSurface	Surface functions
	\ingroup	grpVideo
*/

/*! \defgroup	grpSChr4	4bpp tiled surfaces
	\ingroup	grpSurface
*/

/*! \defgroup	grpSBmp16	16bpp bitmap surfaces
	\ingroup	grpSurface
*/

/*! \defgroup	grpSBmp8	8bpp bitmap surfaces
	\ingroup	grpSurface
*/

/*! \addtogroup grpSurface	*/
/*!	\{	*/

// --------------------------------------------------------------------
// CLASSES
// --------------------------------------------------------------------

//! Surface types
typedef enum ESurfaceType
{
	SRF_NONE		= 0,	//!< No specific type. 
	SRF_CHR4		= 1,	//!< 4bpp tiles.
	SRF_CHR8		= 2,	//!< 8bpp tiles.
	SRF_BMP8		= 3,	//!< 8bpp linear (bitmap/tilemap).
	SRF_BMP16		= 4,	//!< 16bpp linear (bitmap/tilemap).
	//SRF_SBB		= 4		//!< 16bpp tilemap in screenblocks
	SRF_ALLOCATED	= 0x80	//!< Pointers have been allocated.
} ESurfaceType;


typedef struct TSurface
{
	u8	*data;		//!< Surface data pointer.
	u32	pitch;		//!< Scanline pitch in bytes (PONDER: alignment?).
	u16 width;		//!< Image width in pixels.	
	u16 height;		//!< Image width in pixels.
	u8	bpp;		//!< Bits per pixel.
	u8	type;		//!< Surface type (not used that much).
	u16	palSize;	//!< Number of colors.
	u16	*palData;	//!< Pointer to palette.
} TSurface;

/*!	\}	*/

// --------------------------------------------------------------------
// GLOBALS
// --------------------------------------------------------------------

extern const TSurface m3_surface;
extern EWRAM_DATA TSurface m4_surface;
extern EWRAM_DATA TSurface m5_surface;

// --------------------------------------------------------------------
// PROTOTYPES
// --------------------------------------------------------------------

/*! \addtogroup grpSurface	*/
/*!	\{	*/

void srf_init(TSurface *srf, enum ESurfaceType type, const void *data, 
	uint width, uint height, uint bpp, u16 *pal);

void *srf_get_ptr(TSurface *srf, uint x, uint y);

INLINE void srf_set_ptr(TSurface *srf, const void *ptr);
INLINE void srf_set_pal(TSurface *srf, const u16 *pal, uint size);

/*!	\}	*/


/*! \addtogroup grpSChr4	*/
/*!	\{	*/

void schr4_prep_map(const TSurface *srf, u16 *map, u16 se0);
u32 *schr4_get_ptr(const TSurface *srf, int x, int y);

u32 schr4_get_pixel(const TSurface *src, int x, int y);

void schr4_plot(const TSurface *dst, int x, int y, u32 clr);
void schr4_hline(const TSurface *dst, int x1, int y, int x2, u32 clr);
void schr4_vline(const TSurface *dst, int x, int y1, int y2, u32 clr);
void schr4_line(const TSurface *dst, int x1, int y1, int x2, int y2, u32 clr);

void schr4_rect(const TSurface *dst, 
	int left, int top, int right, int bottom, u32 clr);
void schr4_frame(const TSurface *dst, 
	int left, int top, int right, int bottom, u32 clr);

//void schr4_blit(const TSurface *dst, int dstX, int dstY, 
//	uint width, uint height, const TSurface *src, int srcX, int srcY);
//void schr4_floodfill(const TSurface *dst, int x, int y, u32 clr);

/*!	\}	*/



/*! \addtogroup grpSBmp16	*/
/*!	\{	*/

u32  sbmp16_get_pixel(const TSurface *src, int x, int y);

void sbmp16_plot(const TSurface *dst, int x, int y, u32 clr);
void sbmp16_hline(const TSurface *dst, int x1, int y, int x2, u32 clr);
void sbmp16_vline(const TSurface *dst, int x, int y1, int y2, u32 clr);
void sbmp16_line(const TSurface *dst, int x1, int y1, int x2, int y2, u32 clr);

void sbmp16_rect(const TSurface *dst, 
	int left, int top, int right, int bottom, u32 clr);
void sbmp16_frame(const TSurface *dst, 
	int left, int top, int right, int bottom, u32 clr);
void sbmp16_blit(const TSurface *dst, int dstX, int dstY, 
	uint width, uint height, const TSurface *src, int srcX, int srcY);
void sbmp16_floodfill(const TSurface *dst, int x, int y, u32 clr);

/*!	\}	*/


/*! \addtogroup grpSBmp8	*/
/*!	\{	*/

u32  sbmp8_get_pixel(const TSurface *src, int x, int y);

void sbmp8_plot(const TSurface *dst, int x, int y, u32 clr);
void sbmp8_hline(const TSurface *dst, int x1, int y, int x2, u32 clr);
void sbmp8_vline(const TSurface *dst, int x, int y1, int y2, u32 clr);
void sbmp8_line(const TSurface *dst, int x1, int y1, int x2, int y2, u32 clr);

void sbmp8_rect(const TSurface *dst, 
	int left, int top, int right, int bottom, u32 clr);
void sbmp8_frame(const TSurface *dst, 
	int left, int top, int right, int bottom, u32 clr);
void sbmp8_blit(const TSurface *dst, int dstX, int dstY, 
	uint width, uint height, const TSurface *src, int srcX, int srcY);
void sbmp8_floodfill(const TSurface *dst, int x, int y, u32 clr);

/*!	\}	*/


// --------------------------------------------------------------------
// INLINES
// --------------------------------------------------------------------


//! Set Data-pointer surface for \a srf.
INLINE void srf_set_ptr(TSurface *srf, const void *ptr)
{	srf->data= (void*)ptr;						}


//! Set the palette and its size.
INLINE void srf_set_pal(TSurface *srf, const u16 *pal, uint size)
{	
	srf->palSize= size;
	srf->palData= (u16*)pal;
}



#endif // TONC_SURFACE

// EOF
